"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShortcutRegistry = void 0;
const debug_1 = __importDefault(require("debug"));
const events_1 = require("events");
const key_codes_1 = require("./key-codes");
const accelerator_parser_1 = require("./accelerator-parser");
const utils_1 = require("./utils");
class ShortcutRegistry {
    constructor(options) {
        var _a, _b;
        this.parser = new accelerator_parser_1.AcceleratorParser();
        this.eventEmitter = new events_1.EventEmitter();
        this.shortcutRegistered = [];
        this.modifiersPressed = [];
        this.normalKeysPressed = [];
        this.options = options !== null && options !== void 0 ? options : {};
        this.options.strict = (_a = this.options.strict) !== null && _a !== void 0 ? _a : true;
        this.options.filter = (_b = this.options.filter) !== null && _b !== void 0 ? _b : ShortcutRegistry.defaultFilter;
        if (this.options.debug) {
            if (typeof this.options.debug === 'function') {
                this.debug = this.options.debug;
            }
            else {
                this.debug = (0, debug_1.default)('ReactUseShortcuts');
                debug_1.default.enable('ReactUseShortcuts');
            }
        }
        else {
            this.debug = utils_1.noop;
        }
    }
    static keyCodeIsModifiers(keycode) {
        return ShortcutRegistry.KeyCodeModifiers.has(keycode);
    }
    registerShortcut(accelerator, callback) {
        try {
            const [modifiers, normalKeys] = this.parser.parseAccelerator(accelerator);
            const matchShortcut = this.matchShortcut(modifiers, normalKeys);
            if (matchShortcut) {
                this.debug(`Shortcut ${accelerator} conflict with ${matchShortcut.accelerator}`);
                return false;
            }
            this.shortcutRegistered.push({
                accelerator,
                modifiers,
                normalKeys,
                callback,
                enabled: true,
            });
            return true;
        }
        catch (e) {
            this.debug(e.message);
            return false;
        }
    }
    unregisterShortcut(accelerator) {
        try {
            const shortcutRegister = this.matchShortcut(...this.parser.parseAccelerator(accelerator));
            if (!shortcutRegister) {
                this.debug(`Shortcut ${accelerator} is not register yet.`);
                return false;
            }
            this.shortcutRegistered = this.shortcutRegistered.filter((item) => item !== shortcutRegister);
            return true;
        }
        catch (e) {
            this.debug(e.message);
            return false;
        }
    }
    disableShortcut(accelerator) {
        try {
            const [modifiers, normalKeys] = this.parser.parseAccelerator(accelerator);
            const matchShortcut = this.matchShortcut(modifiers, normalKeys);
            if (!matchShortcut) {
                this.debug(`Shortcut ${accelerator} is not register yet.`);
                return false;
            }
            matchShortcut.enabled = false;
            return true;
        }
        catch (e) {
            this.debug(e.message);
            return false;
        }
    }
    enableShortcut(accelerator) {
        try {
            const [modifiers, normalKeys] = this.parser.parseAccelerator(accelerator);
            const matchShortcut = this.matchShortcut(modifiers, normalKeys);
            if (!matchShortcut) {
                this.debug(`Shortcut ${accelerator} is not register yet.`);
                return false;
            }
            matchShortcut.enabled = true;
            return true;
        }
        catch (e) {
            this.debug(e.message);
            return false;
        }
    }
    isShortcutRegistered(accelerator) {
        try {
            return !!this.matchShortcut(...this.parser.parseAccelerator(accelerator));
        }
        catch (e) {
            this.debug(e.message);
            return false;
        }
    }
    getCurrentKeyPressed() {
        return [
            ...this.modifiersPressed
                .map((item) => {
                if (!this.options.strict && ShortcutRegistry.keyCodeIsModifiers(item)) {
                    return ShortcutRegistry.LooseModifiersName.get(item);
                }
                return key_codes_1.keyCode2KeyCodeName.get(item);
            })
                .reduce((memo, item) => {
                if (memo.includes(item)) {
                    return memo;
                }
                memo.push(item);
                return memo;
            }, []),
            ...this.normalKeysPressed.map((item) => key_codes_1.keyCode2KeyCodeName.get(item)),
        ].join(accelerator_parser_1.AcceleratorParser.separator);
    }
    onKeydown(cb) {
        this.eventEmitter.addListener('keydown', cb);
        return () => {
            this.eventEmitter.removeListener('keydown', cb);
        };
    }
    onKeyup(cb) {
        this.eventEmitter.addListener('keyup', cb);
        return () => {
            this.eventEmitter.removeListener('keyup', cb);
        };
    }
    matchShortcut(modifiers, normalKeys) {
        return this.shortcutRegistered.find((item) => {
            const modifiersSet = new Set([...item.modifiers, ...modifiers]);
            const isModifierMatch = modifiersSet.size < item.modifiers.length + modifiers.length;
            const isNormalKeyMatch = normalKeys.find((normalKey) => {
                return item.normalKeys.find((itemNormalKey) => {
                    return itemNormalKey.includes(normalKey) || normalKey.includes(itemNormalKey);
                });
            });
            return isModifierMatch && isNormalKeyMatch;
        });
    }
    attachElement(ele) {
        const handleKeydown = this.handleKeydown.bind(this);
        const handleKeyup = this.handleKeyup.bind(this);
        const clear = this.clear.bind(this);
        ele.addEventListener('keydown', handleKeydown);
        ele.addEventListener('keyup', handleKeyup);
        // window will unfocus when some system global shortcut triggered.
        ele.addEventListener('blur', clear);
        return () => {
            ele.removeEventListener('keydown', handleKeydown);
            ele.removeEventListener('keyup', handleKeyup);
            ele.removeEventListener('blur', clear);
            this.clear();
            this.shortcutRegistered = [];
        };
    }
    handleKeydown(event) {
        if (!this.options.filter(event))
            return;
        const keycode = event.code;
        if (!key_codes_1.KeyCodesSupported.includes(keycode)) {
            this.debug(`Unsupported keyCode: ${event.code}!`);
            return;
        }
        if (ShortcutRegistry.keyCodeIsModifiers(keycode)) {
            this.modifiersPressed.push(keycode);
            // reset keycode record when modifiers change
            this.normalKeysPressed = [];
        }
        else {
            this.normalKeysPressed.push(keycode);
        }
        this.triggerShortcutEventIfHandlerFound(event);
        this.eventEmitter.emit('keydown', event);
    }
    handleKeyup(event) {
        if (!this.options.filter(event))
            return;
        const keycode = event.code;
        if (ShortcutRegistry.keyCodeIsModifiers(keycode)) {
            this.modifiersPressed = this.modifiersPressed.filter((code) => {
                return keycode !== code;
            });
            // reset keycode record when modifiers change
            this.normalKeysPressed = [];
        }
        this.eventEmitter.emit('keyup', event);
    }
    clear() {
        this.modifiersPressed = [];
        this.normalKeysPressed = [];
    }
    triggerShortcutEventIfHandlerFound(event) {
        const modifier = [...this.modifiersPressed].sort().join(accelerator_parser_1.AcceleratorParser.separator);
        const normalKey = this.normalKeysPressed.join(accelerator_parser_1.AcceleratorParser.separator);
        const shortcutRegister = this.shortcutRegistered.find((item) => {
            return (item.modifiers.includes(modifier) && item.normalKeys.find((itemNormalKey) => normalKey.endsWith(itemNormalKey)));
        });
        if (shortcutRegister) {
            if (shortcutRegister.enabled) {
                shortcutRegister.callback(event);
            }
            this.normalKeysPressed = [];
        }
    }
}
exports.ShortcutRegistry = ShortcutRegistry;
ShortcutRegistry.KeyCodeModifiers = new Set([
    'ControlLeft',
    'ControlRight',
    'ShiftLeft',
    'ShiftRight',
    'AltLeft',
    'AltRight',
    'MetaLeft',
    'MetaRight',
]);
ShortcutRegistry.LooseModifiersName = new Map([
    ['ControlLeft', 'Ctrl'],
    ['ControlRight', 'Ctrl'],
    ['AltLeft', 'Alt'],
    ['AltRight', 'Alt'],
    ['ShiftLeft', 'Shift'],
    ['ShiftRight', 'Shift'],
    ['MetaLeft', 'Meta'],
    ['MetaRight', 'Meta'],
]);
ShortcutRegistry.defaultFilter = (event) => {
    if (event.repeat)
        return false;
    if (event.target && event.target instanceof HTMLElement) {
        return !['INPUT', 'TEXTAREA', 'SELECT'].includes(event.target.tagName) && !event.target.isContentEditable;
    }
    return true;
};
//# sourceMappingURL=shortcut-registry.js.map